@@ -65,9 +65,6 @@ class Parser {
     };
     */
 
-    /** the supported JNLP file versions */
-    private static Version supportedVersions = new Version("1.0 1.1 1.2 1.3 1.4 1.5 1.6 6.0");
-
     // fix: some descriptors need to use the jnlp file at a later
     // date and having file ref lets us pass it to their
     // constructors
@@ -128,21 +125,10 @@ class Parser {
         this.base = (codebase != null) ? codebase : base; // if codebase not specified use default codebase
         fileLocation = getURL(root, "href", this.base);
 
-        // ensure version is supported
-        if (!supportedVersions.matchesAny(spec))
-            throw new ParseException(R("PSpecUnsupported", supportedVersions));
-
         // normalize the text nodes
         root.normalize();
     }
 
-    /**
-     * Return the JNLP specification versions supported.
-     */
-    public static Version getSupportedVersions() {
-        return supportedVersions;
-    }
-
     /**
      * Returns the file version.
      */
@@ -350,7 +336,6 @@ class Parser {
         String part = getAttribute(node, "part", null);
         boolean main = "true".equals(getAttribute(node, "main", "false"));
         boolean lazy = "lazy".equals(getAttribute(node, "download", "eager"));
-        int size = Integer.parseInt(getAttribute(node, "size", "0"));
 
         if (nativeJar && main)
             if (strict)
@@ -460,9 +445,9 @@ class Parser {
             String name = child.getNodeName();
 
             if ("title".equals(name))
-                addInfo(info, child, null, getSpanText(child));
+                addInfo(info, child, null, getSpanText(child, false));
             if ("vendor".equals(name))
-                addInfo(info, child, null, getSpanText(child));
+                addInfo(info, child, null, getSpanText(child, false));
             if ("description".equals(name)) {
                 String kind = getAttribute(child, "kind", "default");
                 if (descriptionsUsed.contains(kind))
@@ -470,7 +455,7 @@ class Parser {
                         throw new ParseException(R("PTwoDescriptions", kind));
 
                 descriptionsUsed.add(kind);
-                addInfo(info, child, kind, getSpanText(child));
+                addInfo(info, child, kind, getSpanText(child, false));
             }
             if ("homepage".equals(name))
                 addInfo(info, child, null, getRequiredURL(child, "href", base));
@@ -788,12 +773,12 @@ class Parser {
                 if (title != null && strict) {
                     throw new ParseException(R("PTwoTitles"));
                 }
-                title = getSpanText(child);
+                title = getSpanText(child, false);
             } else if ("description".equals(name)) {
                 if (description != null && strict) {
                     throw new ParseException(R("PTwoDescriptions"));
                 }
-                description = getSpanText(child);
+                description = getSpanText(child, false);
             } else if ("icon".equals(name)) {
                 if (icon != null && strict) {
                     throw new ParseException(R("PTwoIcons"));
@@ -890,7 +875,6 @@ class Parser {
     }
 
     // XML junk
-
     /**
      * Returns the implied text under a node, for example "text" in
      * "<description>text</description>".
@@ -899,11 +883,35 @@ class Parser {
      * @throws ParseException if the JNLP file is invalid
      */
     public String getSpanText(Node node) throws ParseException {
+        return getSpanText(node, true);
+    }
+
+    /**
+     * Returns the implied text under a node, for example "text" in
+     * "<description>text</description>". If preserveSpacing is false,
+     * sequences of whitespace characters are turned into a single
+     * space character.
+     *
+     * @param node the node with text under it
+     * @param preserveSpacing if true, preserve whitespace
+     * @throws ParseException if the JNLP file is invalid
+     */
+    public String getSpanText(Node node, boolean preserveSpacing)
+            throws ParseException {
         if (node == null)
             return null;
 
         // NANO
-        return node.getNodeValue();
+        String val = node.getNodeValue();
+        if (preserveSpacing) {
+            return val;
+        } else {
+            if (val == null) {
+                return null;
+            } else {
+                return val.replaceAll("\\s+", " ");
+            }
+        }
 
         /* TINY
         Node child = node.getFirstChild();
