@@ -24,9 +24,7 @@ package net.sourceforge.jnlp;
 
 import java.net.URL;
 import java.net.MalformedURLException;
-import java.util.Calendar;
 import java.util.Hashtable;
-import java.util.Iterator;
 import java.util.Locale;
 import java.util.List;
 import java.util.ArrayList;
@@ -41,6 +39,9 @@ public class PluginBridge extends JNLPFile {
     String[] cacheJars = new String[0];
     String[] cacheExJars = new String[0];
     Hashtable<String, String> atts;
+    private boolean usePack;
+    private boolean useVersion;
+    private boolean codeBaseLookup;
 
     public PluginBridge(URL codebase, URL documentBase, String jar, String main,
                         int width, int height, Hashtable<String, String> atts)
@@ -131,9 +132,63 @@ public class PluginBridge extends JNLPFile {
         else
             security = null;
 
-        this.uniqueKey = Calendar.getInstance().getTimeInMillis() + "-" +
-                         Math.abs(((new java.util.Random()).nextInt())) + "-" +
-                         documentBase;
+        /* According to http://download.oracle.com/javase/6/docs/technotes/guides/deployment/deployment-guide/applet-compatibility.html, 
+         * classloaders are shared iff these properties match:
+         * codebase, cache_archive, java_archive, archive
+         * 
+         * To achieve this, we create the uniquekey based on those 4 values,
+         * always in the same order. The initial "<NAME>=" parts ensure a 
+         * bad tag cannot trick the loader into getting shared with another.
+         */
+
+        // Firefox sometimes skips the codebase if it is default  -- ".", 
+        // so set it that way if absent
+        String codebaseAttr =      atts.get("codebase") != null ?
+                                   atts.get("codebase") : ".";
+
+        String cache_archiveAttr = atts.get("cache_archive") != null ? 
+                                   atts.get("cache_archive") : "";
+
+        String java_archiveAttr =  atts.get("java_archive") != null ? 
+                                   atts.get("java_archive") : "";
+
+        String archiveAttr =       atts.get("archive") != null ? 
+                                   atts.get("archive") : "";
+
+        this.uniqueKey = "codebase=" + codebaseAttr +
+                         "cache_archive=" + cache_archiveAttr + 
+                         "java_archive=" + java_archiveAttr + 
+                         "archive=" +  archiveAttr;
+
+        usePack = false;
+        useVersion = false;
+        String jargs = atts.get("java_arguments");
+        if (jargs != null) {
+            for (String s : jargs.split(" ")) {
+                String[] parts = s.trim().split("=");
+                if (parts.length == 2 && Boolean.valueOf(parts[1])) {
+                    if ("-Djnlp.packEnabled".equals(parts[0])) {
+                        usePack = true;
+                    } else if ("-Djnlp.versionEnabled".equals(parts[0])) {
+                        useVersion = true;
+                    }
+                }
+            }
+        }
+        String cbl = atts.get("codebase_lookup");
+        codeBaseLookup = cbl == null || (Boolean.valueOf(cbl));
+    }
+
+    public boolean codeBaseLookup() {
+    	return codeBaseLookup;
+    }
+
+    /**
+     * {@inheritdoc }
+     */
+    @Override
+    public DownloadOptions getDownloadOptionsForJar(JARDesc jar) {
+        return new DownloadOptions(usePack, useVersion);
     }
 
     public String getTitle() {
@@ -245,6 +300,14 @@ public class PluginBridge extends JNLPFile {
         };
     }
 
+    /**
+     * Returns the resources section of the JNLP file for the
+     * specified locale, os, and arch.
+     */
+    public ResourcesDesc[] getResourcesDescs(final Locale locale, final String os, final String arch) {
+        return new ResourcesDesc[] { getResources(locale, os, arch) };
+    }
+
     public boolean isApplet() {
         return true;
     }
