@@ -28,6 +28,8 @@ import java.net.HttpURLConnection;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLConnection;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.concurrent.ConcurrentHashMap;
@@ -673,7 +675,7 @@ public class ResourceTracker {
             CacheEntry downloadEntry = new CacheEntry(downloadLocation, resource.downloadVersion);
             File finalFile = CacheUtil.getCacheFile(resource.location, resource.downloadVersion); // This is where extracted version will be, or downloaded file if not compressed.
 
-            if (!finalFile.exists()) {
+            if (!downloadEntry.isCurrent(con)) {
                 // Make sure we don't re-download the file. however it will wait as if it was downloading.
                 // (This is fine because file is not ready yet anyways)
                 byte buf[] = new byte[1024];
@@ -698,6 +700,7 @@ public class ResourceTracker {
                  * If the file was compressed, uncompress it.
                  */
                 if (packgz) {
+                    downloadEntry.initialize(con);
                     GZIPInputStream gzInputStream = new GZIPInputStream(new FileInputStream(CacheUtil
                             .getCacheFile(downloadLocation, resource.downloadVersion)));
                     InputStream inputStream = new BufferedInputStream(gzInputStream);
@@ -712,6 +715,7 @@ public class ResourceTracker {
                     inputStream.close();
                     gzInputStream.close();
                 } else if (gzip) {
+                    downloadEntry.initialize(con);
                     GZIPInputStream gzInputStream = new GZIPInputStream(new FileInputStream(CacheUtil
                             .getCacheFile(downloadLocation, resource.downloadVersion)));
                     InputStream inputStream = new BufferedInputStream(gzInputStream);
@@ -1078,7 +1082,7 @@ public class ResourceTracker {
     /**
      * This class downloads and initializes the queued resources.
      */
-    class Downloader implements Runnable {
+    private class Downloader implements Runnable {
         Resource resource = null;
 
         public void run() {
@@ -1100,7 +1104,19 @@ public class ResourceTracker {
                 }
 
                 try {
-                    processResource(resource);
+
+                    // Resource processing involves writing to files 
+                    // (cache entry trackers, the files themselves, etc.)
+                    // and it therefore needs to be privileged
+
+                    final Resource fResource = resource;
+                    AccessController.doPrivileged(new PrivilegedAction<Void>() {
+                        public Void run() {
+                            processResource(fResource);                            
+                            return null;
+                        }
+                    });
+
                 } catch (Exception ex) {
                     if (JNLPRuntime.isDebug())
                         ex.printStackTrace();
