@@ -130,11 +130,12 @@ PluginRequestProcessor::newMessageOnBus(const char* message)
                    !command->find("GetSlot") ||
                    !command->find("SetSlot") ||
                    !command->find("Eval") ||
-                   !command->find("Finalize"))
+                   !command->find("Finalize") ||
+                   !command->find("LoadURL"))
         {
 
             // Update queue synchronously
-        	pthread_mutex_lock(&message_queue_mutex);
+            pthread_mutex_lock(&message_queue_mutex);
             message_queue->push_back(message_parts);
             pthread_mutex_unlock(&message_queue_mutex);
 
@@ -238,20 +239,7 @@ PluginRequestProcessor::eval(std::vector<std::string*>* message_parts)
     thread_data.parameters.push_back(NPVARIANT_TO_OBJECT(*window_ptr));
     thread_data.parameters.push_back(&script);
 
-#ifdef CHROMIUM_WORKAROUND
-    // Workaround for chromium
-    _eval(&thread_data);
-
-    if (!thread_data.call_successful)
-    {
-#endif
-        thread_data.result_ready = false;
-        browser_functions.pluginthreadasynccall(instance, &_eval, &thread_data);
-
-        while (!thread_data.result_ready) usleep(2000); // Wait till result is ready
-#ifdef CHROMIUM_WORKAROUND
-    }
-#endif
+    IcedTeaPluginUtilities::callAndWaitForResult(instance, &_eval, &thread_data);
 
     NPVariant* result_variant = (NPVariant*) IcedTeaPluginUtilities::stringToJSID(thread_data.result);
     std::string result_variant_jniid = std::string();
@@ -340,20 +328,7 @@ PluginRequestProcessor::call(std::vector<std::string*>* message_parts)
     thread_data.parameters.push_back(&arg_count);
     thread_data.parameters.push_back(args_array);
 
-#ifdef CHROMIUM_WORKAROUND
-    // Workaround for chromium
-    _call(&thread_data);
-
-    if (!thread_data.call_successful)
-    {
-#endif
-        thread_data.result_ready = false;
-        browser_functions.pluginthreadasynccall(instance, &_call, &thread_data);
-
-        while (!thread_data.result_ready) usleep(2000); // wait till ready
-#ifdef CHROMIUM_WORKAROUND
-    }
-#endif
+    IcedTeaPluginUtilities::callAndWaitForResult(instance, &_call, &thread_data);
 
     result_variant = (NPVariant*) IcedTeaPluginUtilities::stringToJSID(thread_data.result);
 
@@ -408,19 +383,7 @@ PluginRequestProcessor::sendString(std::vector<std::string*>* message_parts)
     thread_data.parameters.push_back(instance);
     thread_data.parameters.push_back(variant);
 
-#ifdef CHROMIUM_WORKAROUND
-    // Workaround for chromium
-    _getString(&thread_data);
-
-    if (!thread_data.call_successful)
-    {
-#endif
-        thread_data.result_ready = false;
-        browser_functions.pluginthreadasynccall(instance, &_getString, &thread_data);
-        while (!thread_data.result_ready) usleep(2000); // wait till ready
-#ifdef CHROMIUM_WORKAROUND
-    }
-#endif
+    IcedTeaPluginUtilities::callAndWaitForResult(instance, &_getString, &thread_data);
 
     // We need the context 0 for backwards compatibility with the Java side
     IcedTeaPluginUtilities::constructMessagePrefix(0, reference, &response);
@@ -501,20 +464,7 @@ PluginRequestProcessor::setMember(std::vector<std::string*>* message_parts)
     thread_data.parameters.push_back(&property_identifier);
     thread_data.parameters.push_back(&value);
 
-#ifdef CHROMIUM_WORKAROUND
-    // Workaround for chromium
-    _setMember(&thread_data);
-
-    if (!thread_data.call_successful)
-    {
-#endif
-        thread_data.result_ready = false;
-        browser_functions.pluginthreadasynccall(instance, &_setMember, &thread_data);
-
-        while (!thread_data.result_ready) usleep(2000); // wait till ready
-#ifdef CHROMIUM_WORKAROUND
-    }
-#endif
+    IcedTeaPluginUtilities::callAndWaitForResult(instance, &_setMember, &thread_data);
 
     IcedTeaPluginUtilities::constructMessagePrefix(0, reference, &response);
     response.append(" JavaScriptSetMember ");
@@ -597,21 +547,7 @@ PluginRequestProcessor::sendMember(std::vector<std::string*>* message_parts)
     thread_data.parameters.push_back(NPVARIANT_TO_OBJECT(*parent_ptr));
     thread_data.parameters.push_back(&member_identifier);
 
-#ifdef CHROMIUM_WORKAROUND
-    // Workaround for chromium
-    _getMember(&thread_data);
-
-    if (!thread_data.call_successful)
-    {
-#endif
-        thread_data.result_ready = false;
-        browser_functions.pluginthreadasynccall(instance, &_getMember, &thread_data);
-
-        while (!thread_data.result_ready) usleep(2000); // wait till ready
-
-#ifdef CHROMIUM_WORKAROUND
-    }
-#endif
+    IcedTeaPluginUtilities::callAndWaitForResult(instance, &_getMember, &thread_data);
 
     PLUGIN_DEBUG("Member PTR after internal request: %s\n", thread_data.result.c_str());
 
@@ -714,6 +650,37 @@ PluginRequestProcessor::finalize(std::vector<std::string*>* message_parts)
     plugin_to_java_bus->post(response.c_str());
 }
 
+/**
+ * Fetches the URL and loads it into the given target
+ *
+ * @param message_parts The request message.
+ */
+void
+PluginRequestProcessor::loadURL(std::vector<std::string*>* message_parts)
+{
+
+    int id = atoi(message_parts->at(1)->c_str());
+
+    AsyncCallThreadData thread_data = AsyncCallThreadData();
+    thread_data.result_ready = false;
+    thread_data.parameters = std::vector<void*>();
+    thread_data.result = std::string();
+
+    NPP instance;
+    get_instance_from_id(id, instance);
+
+    // If instance is invalid, do not proceed further
+    if (!instance)
+    	return;
+
+    thread_data.parameters.push_back(instance);
+    thread_data.parameters.push_back(message_parts->at(5)); // push url
+    thread_data.parameters.push_back(message_parts->at(6)); // push target
+
+    thread_data.result_ready = false;
+    IcedTeaPluginUtilities::callAndWaitForResult(instance, &_loadURL, &thread_data);
+}
+
 static void
 queue_cleanup(void* data)
 {
@@ -794,6 +761,12 @@ queue_processor(void* data)
                 pthread_mutex_lock(&syn_write_mutex);
                 processor->finalize(message_parts);
                 pthread_mutex_unlock(&syn_write_mutex);
+            } else if (command == "LoadURL") // For instance X url <url> <target>
+            {
+                // write methods are synchronized
+                pthread_mutex_lock(&syn_write_mutex);
+                processor->loadURL(message_parts);
+                pthread_mutex_unlock(&syn_write_mutex);
             } else
             {
                 // Nothing matched
@@ -1014,3 +987,34 @@ _getString(void* data)
     PLUGIN_DEBUG("_getString returning\n");
 }
 
+void
+_loadURL(void* data) {
+
+    PLUGIN_DEBUG("_loadURL called\n");
+
+    NPP instance;
+    std::string* url;
+    std::string* target;
+
+    std::vector<void*> parameters = ((AsyncCallThreadData*) data)->parameters;
+
+    instance = (NPP) parameters.at(0);
+    url = (std::string*) parameters.at(1);
+    target = (std::string*) parameters.at(2);
+
+    PLUGIN_DEBUG("Launching %s in %s\n", url->c_str(), target->c_str());
+
+    // Each decode can expand to 4 chars at the most
+    gchar* decoded_url = (gchar*) calloc(strlen(url->c_str())*4 + 1, sizeof(gchar));
+    IcedTeaPluginUtilities::decodeURL(url->c_str(), &decoded_url);
+
+    ((AsyncCallThreadData*) data)->call_successful =
+        (*browser_functions.geturl) (instance, decoded_url, target->c_str());
+
+    ((AsyncCallThreadData*) data)->result_ready = true;
+
+    g_free(decoded_url);
+    decoded_url = NULL;
+
+    PLUGIN_DEBUG("_loadURL returning %d\n", ((AsyncCallThreadData*) data)->call_successful);
+}
